-----------------------------------------------------------------------------
-- EuMIDI for DOS32
-----------------------------------------------------------------------------
--
-- Source code copyright (C) Stardust Entertainment, 2000.
-- http://www2.50megs.com/sonic/
--
-- The source code is available for distribution and/or modification only
-- under the terms that it always remain freely available.
--
-- The source is distributed in the hope it will be useful, but WITHOUT
-- ANY WARRANTY; without even the implied warranty of FITNESS FOR A
-- PARTICUALAR PURPOSE. In other words, don't come knocking at my door
-- when the pink elephants start coming out of your monitor.
--
-- 27-03-2000, 17:03
--
-- v0.01.000 update: 27-03-2000, 17:03
--
-----------------------------------------------------------------------------
--
-- EuMIDI for DOS32
-- An include file to allow Euphoria programs to play MIDI files in the
-- background. The SBMIDI and SBSIM device drivers must be loaded before
-- the program can be used. Requires a 100% SoundBlaster compatible sound
-- card.
--
-- Before this modules can be used, you must first load the SBMIDI file
-- followed by the SBSIM file. It is best to invoke these in a batch file.
-- To unload these drivers, simply use the /u parameter.
--
--    @ECHO OFF
--    SBMIDI
--    SBSIM
--    ex myprog.ex
--    SBSIM /u
--    SBMIDI /u
--
-- Peter Millan
-- http://www2.50megs.com/sonic/
-- sonic@globalfreeway.com.au
-----------------------------------------------------------------------------

-- Needed by the modules.
include get.e
include machine.e

global constant
  mid_interrupt = #81        -- this is usually always the same, but can
                             -- be changed if necessary. this is the value
                             -- sbsim returns.

-----------------------------------------------------------------------------
-----------------------------------------------------------------------------
global procedure mid_stop()
  sequence
    mid_regs,
    mid_regsret

  mid_regs = repeat(0, 10)
  mid_regs[REG_BX] = #502
  mid_regsret = dos_interrupt(mid_interrupt, mid_regs)
end procedure

-----------------------------------------------------------------------------
-----------------------------------------------------------------------------
global procedure mid_pause()
  sequence
    mid_regs,
    mid_regsret

  mid_regs = repeat(0, 10)
  mid_regs[REG_BX] = #503
  mid_regsret = dos_interrupt(mid_interrupt, mid_regs)
end procedure

-----------------------------------------------------------------------------
-----------------------------------------------------------------------------
global procedure mid_resume()
  sequence
    mid_regs,
    mid_regsret

  mid_regs = repeat(0, 10)
  mid_regs[REG_BX] = #504
  mid_regsret = dos_interrupt(mid_interrupt, mid_regs)
end procedure

-----------------------------------------------------------------------------
-----------------------------------------------------------------------------
global function mid_soundstatus()
  sequence
    mid_regs,
    mid_regsret

  mid_regs = repeat(0, 10)
  mid_regs[REG_BX] = #505
  mid_regsret = dos_interrupt(mid_interrupt, mid_regs)
  return mid_regsret[REG_AX]
end function


-----------------------------------------------------------------------------
-----------------------------------------------------------------------------
global function mid_loadmidi(sequence filename)
  sequence
    mid_regs,
    mid_regsret
  atom
    mid_memloc

  -- allocate the length of the filename + 1 in conventional memory.
  mid_memloc = allocate_low(length(filename)+1)

  -- poke in the filename, and the NULL character (needed for the driver)
  poke(mid_memloc, filename & 0)

  -- allocate space for the registers
  mid_regs = repeat(0, 10)
  mid_regs[REG_BX] = #500
  mid_regs[REG_AX] = mid_memloc
  mid_regsret = dos_interrupt(mid_interrupt, mid_regs)

  -- Ooops. MID was not loaded succesfully. Let the programmer decide what
  -- to do by returning 1.
  if mid_regsret[REG_AX] != 0 then
    return 1
  end if

  -- Success!
  return 0
end function

-----------------------------------------------------------------------------
-----------------------------------------------------------------------------
global function mid_playmidi()
  sequence
    mid_regs,
    mid_regsret

  -- Now we set it up to play.
  mid_regs = repeat(0, 10)
  mid_regs[REG_BX] = #501
  mid_regs[REG_BP] = 1
  mid_regsret = dos_interrupt(mid_interrupt, mid_regs)

  -- Ooops #2. MID could not be played for some reason.
  if mid_regsret[REG_AX] != 0 then
    return 2
  end if

  -- Success!
  return 0
end function

-----------------------------------------------------------------------------
-----------------------------------------------------------------------------
global function mid_loadandplay(sequence filename)
  if mid_loadmidi(filename) != 0 then return 1 end if
  if mid_playmidi() != 0 then return 2 end if
  return 0
end function
